use actix_web::{error, web, HttpResponse};
use handlebars::Handlebars;
use lettre::{AsyncSmtpTransport, Tokio1Executor};
use serde_json::json;
use terminwahl_typen::IdType;

use crate::{
    db::{
        write::{cancel_appointment, confirm_appointments},
        Pool,
    },
    CssPath,
};

pub async fn confirm_validation_key(
    pool: web::Data<Pool>,
    _mailer: web::Data<AsyncSmtpTransport<Tokio1Executor>>,
    css: web::Data<CssPath>,
    handlebars: web::Data<Handlebars<'_>>,
    validation_key: web::Path<String>,
) -> Result<HttpResponse, error::Error> {
    match confirm_appointments(&pool, &validation_key).await {
        Ok(appointments) => Ok({
            let data = json!({
                "css_file" :  css.path,
                "appointments": appointments,
                "validation_key": validation_key.into_inner(),
            });
            HttpResponse::Ok().body(handlebars.render("confirmed.html", &data).unwrap())
        }),
        Err(e) => Err(error::ErrorBadRequest(e)),
    }
}

pub async fn delete_appointment(
    pool: web::Data<Pool>,
    _mailer: web::Data<AsyncSmtpTransport<Tokio1Executor>>,
    handlebars: web::Data<Handlebars<'_>>,
    css: web::Data<CssPath>,
    path: web::Path<(IdType, IdType, String)>,
) -> Result<HttpResponse, error::Error> {
    let (teacher_id, slot_id, validation_key) = path.into_inner();
    match cancel_appointment(&pool, teacher_id, slot_id, &validation_key).await {
        Ok(appointments) => {
            let data = json!({
                "css_file" :  css.path,
                "appointments": appointments,
                "validation_key": validation_key,
            });
            Ok(HttpResponse::Ok().body(handlebars.render("confirmed.html", &data).unwrap()))
        }
        Err(e) => Err(error::ErrorBadRequest(e)),
    }
}
